require 'spec_helper'

describe Rewire::Pipe do
  it 'runs a simple pipe' do
    upcaser = Rewire::Pipe.new { |out, packet| out << packet.upcase }

    output = []
    pipe = upcaser | output

    pipe << 'test'
    pipe << 'another test'

    expect(output).to eq ['TEST', 'ANOTHER TEST']
  end

  it 'creates its own output (sink) when none is provided' do
    pipe = Rewire::Pipe.new { |out, packet| out << packet }
    output = pipe << 'test'
    expect(output).to eq ['test']
  end

  it 'outputs empty array if no "emission" (filter)' do
    output = []
    pipe = Rewire::Pipe.new do |out, packet|
      # no emission
    end | output
    pipe << 'test'
    expect(output).to eq []
  end

  it 'emits can be used more times in a single Pipe' do
    output = []
    pipe = Rewire::Pipe.new do |out, packet|
      out << packet
      out << packet
    end | output
    pipe << 'test'
    expect(output).to eq ['test', 'test']
  end

  it 'creates a pipe using the "Pipe" function' do
    pipe = Pipe { |out, packet| }
    expect(pipe.class).to eq Rewire::Pipe
  end

  it 'connects two pipes' do
    output = []
    pipe = Pipe { |out, packet| out << packet.upcase } | Pipe { |out, packet| out << [packet.size, packet] } | output
    pipe << 'test'
    expect(output).to eq [[4, 'TEST']]
  end

  it 'creates its own output (sink) even with connected pipes' do
    pipe = Pipe { |out, packet| out << packet.upcase } | Pipe { |out, packet| out << [packet.size, packet] }
    output = pipe << 'test'
    expect(output).to eq [[4, 'TEST']]
  end

  it 'run iteratively' do
    source = [1, 2, 3]
    sink = []

    pipe =
      Pipe { |out, n| out << n * 2 } |
      Pipe { |out, n| out << n + 1 } |
      sink

    source.each { |it| pipe << it }

    expect(sink).to eq [3, 5, 7]
  end

  it 'compounds immutable pipes (they can be reused)' do
    pipe_a =
      Pipe { |out, n| out << n * 2 } |
      Pipe { |out, n| out << n + 1 }

    pipe_b =
      Pipe { |out, n| out << n + 2 }

    compound_pipe_1 = pipe_a | pipe_b
    compound_pipe_2 = pipe_b | pipe_a

    expect(compound_pipe_1 << 3).to eq [9]
    expect(compound_pipe_2 << 3).to eq [11]
  end
end
